/* $Id: ChatCore.h 2457 2012-03-29 08:02:24Z IMPOMEZIA $
 * IMPOMEZIA Simple Chat
 * Copyright © 2008-2012 IMPOMEZIA <schat@impomezia.com>
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CHATCORE_H_
#define CHATCORE_H_

#include <QIcon>
#include <QObject>
#include <QUrl>
#include <QVariant>

#include "schat.h"

class ChatPlugins;
class ChatSettings;
class FileLocations;
class NetworkManager;
class SimpleClient;
class Translation;

#define SCHAT_OPTION(x) ChatCore::settings()->value(x)

class SCHAT_CORE_EXPORT ChatCore : public QObject
{
  Q_OBJECT

public:
  ChatCore(QObject *parent = 0);
  ~ChatCore();
  inline static ChatCore *i()                           { return m_self; }
  inline static ChatSettings *settings()                { return m_self->m_settings; }
  inline static FileLocations *locations()              { return m_self->m_locations; }
  inline static NetworkManager *networks()              { return m_self->m_networkManager; }
  inline static QByteArray currentId()                  { return m_self->m_currentId; }
  inline static Translation *translation()              { return m_self->m_translation; }
  inline static void setCurrentId(const QByteArray &id) { m_self->m_currentId = id; }
  static QByteArray randomId();

public slots:
  void send(const QString &text);

private slots:
  void settingsChanged(const QString &key, const QVariant &value);
  void start();

private:
  void loadTranslation();

  ChatPlugins *m_plugins;                         ///< Загрузчик плагинов.
  ChatSettings *m_settings;                       ///< Настройки.
  FileLocations *m_locations;                     ///< Схема размещения файлов.
  NetworkManager *m_networkManager;               ///< Объект управляющих сетями.
  QByteArray m_currentId;                         ///< Идентификатор текущей вкладки.
  static ChatCore *m_self;                        ///< Указатель на себя.
  Translation *m_translation;                     ///< Модуль загрузки переводов.
};

#endif /* CHATCORE_H_ */
